/********************************************/
/********** Default Button Support **********/
/********************************************/

var ENTER_KEY  =  13;
var ESCAPE_KEY =  27; // ESC
var HELP_KEY   = 112; // F1

var DEFAULT_BUTTON = "isDefault";

/**
 *
 * Determine if ENTER key pressed in an input element.  If so, programmatically
 * "click" the default button, if designated.
 *
 */
function keyPressed( evnt ) {
   var swallowKeyPress = false;
   var evnt = (evnt) ? evnt : ((event) ? event : null);
   if ( evnt ) {
      var source  = (evnt.target) ? evnt.target : ((evnt.srcElement) ? evnt.srcElement : null);
      var keycode = evnt.keyCode;
      /***** ENTER Key processing *****/
      if ( keycode == ENTER_KEY ) {
         var type = ( source.type ) ? source.type.toUpperCase() : "null";
         var id   = ( source.id ) ? source.id : "null";
         if ( type == "TEXT"     || type == "PASSWORD" ||
              type == "CHECKBOX" || type == "RADIO" ) {
            if ( source.form ) {
               var buttons = source.form.elements;
               for ( var i = 0; i < buttons.length; i++ ) {
                  type = buttons[i].type.toUpperCase();
                  if ( type == "BUTTON" || type == "SUBMIT" || type == "IMAGE" ) {
                     if ( buttons[i].getAttribute( DEFAULT_BUTTON ) ) {
                        swallowKeyPress = true;
                        buttons[i].click();
                        break;
                     } // end if
                  } // end if
               } // end for
            } // endif
         } // end if
      } // end if
      /***** ESCAPE Key processing *****/
//    else if ( keycode == ESCAPE_KEY ) {
//       if ( source.form ) {
//          var buttons = source.form.elements;
//          for ( i = 0; i < buttons.length; i++ ) {
//             var value = buttons[i].value.toUpperCase();
//             if ( value == "CANCEL" ) {
//                swallowKeyPress = true;
//                buttons[i].click();
//                break;
//             } // end if
//          } // end for
//       } // endif
//    } // end else if
      /***** F1 Key processing *****/
//    else if ( keycode == HELP_KEY ) {
//       if ( source.form ) {
//          var buttons = source.form.elements;
//          for ( i = 0; i < buttons.length; i++ ) {
//             var value = buttons[i].value.toUpperCase();
//             if ( value == "HELP" ) {
//                swallowKeyPress = true;
//                buttons[i].click();
//                break;
//             } // end if
//          } // end for
//       } // endif
//    } // end else if
   } // end if
   if ( swallowKeyPress ) {
      if ( evnt.returnValue ) {
         evnt.returnValue = false;
      } // end if
      else if ( evnt.preventDefault ) {
         evnt.preventDefault();
      } // end else if
      else {
         return false;
      } // end else
   } // end if
} // keyPressed()

/**
 * Register a handler to process a keypress.
 */ 
function registerDefaultButtonListener() {
   // If browser is IE, then addEventListener method is not supported ...
   if ( document.addEventListener ) {
      document.addEventListener( "keypress", keyPressed, false );
   } // end if
   else {
      document.attachEvent( "onkeypress", keyPressed );
   } // end else
} // registerDefaultButtonListener()


/*******************************************/
/********** Initial Focus Support **********/
/*******************************************/

/** */
function dataObject() {
   this.firstNonSubmit = null;
   this.firstSubmit    = null;
   this.initialFocus   = null;
   this.defaultButton  = null;
   this.allSubmitItems = true;
} // dataObject()

var DEFAULT_FOCUS = "defaultFocus";
var VALUE_HELP    = "HELP";
var TYPE_HIDDEN   = "HIDDEN";
var TYPE_SUBMIT   = "SUBMIT";
var TAG_ANCHOR    = "A";
var TAG_BUTTON    = "BUTTON";
var TAG_INPUT     = "INPUT";
var TAG_SELECT    = "SELECT";
var TAG_TEXTAREA  = "TEXTAREA";
var ATTR_HREF     = "HREF";
var ATTR_READONLY = "READONLY";

var focusElement = null;

/** 
 * 
 * Set the inital focus on the panel.
 * 
 * Priority is given to form elements.  Anchor tags with HREF attributes are
 * given secondary priority.  So, for example, if a panel has a menu and an 
 * input field, the input field will end up with initial focus.  This was a 
 * compromise made in order to speed up processing.
 * 
 * It seems that form.elements does not return INPUT tags with type="IMAGE". 
 * I tired this on Firefox, Internet Explorer and Mozilla.
 * 
 * The revised algorithm should be sufficient for the great majority of panels.
 * One trouble spot will be large tables (1000+ rows of data) that have 
 * selectable rows.  Client-side select tables pose the greatest problem, since
 * all the "select" components are form elements.  Server-side tables have 
 * anchor tags with href attributes.  
 * 
 */
function setInitialPanelFocus() {
   var data = new dataObject();
   if ( document.forms.length > 0 ) {
      evaluateForm( document.forms[0], data );
//    var output = "";
//    if ( data.firstNonSubmit != null ) {
//       output += "First Non-Submit = " + data.firstNonSubmit.tagName + " : " + data.firstNonSubmit.type + " : " + data.firstNonSubmit.id + "\n";
//    } // end if 
//    if ( data.firstSubmit != null ) {
//       output += "First Submit = " + data.firstSubmit.tagName + " : " + data.firstSubmit.value + "\n";
//    } // end if 
//    if ( data.initialFocus != null ) {
//       output += "Initial Focus = " + data.initialFocus.tagName + " : " + data.initialFocus.id + "\n";
//    } // end if 
//    if ( data.defaultButton != null ) {
//       output += "Default Button = " + data.defaultButton.tagName + " : " + data.defaultButton.value + "\n";
//    } // end if 
//    alert( output );
      focusElement = null;
      // Is there a default button?  If so, it has preference over initial focus
      if ( data.defaultButton ) {
         // Only buttons on the panel ... i.e. Confirmation Panel
         if ( data.allSubmitItems ) {
            focusElement = data.defaultButton;
         } // end if
         // This is somewhat counter-intuitive ...
         else if ( data.initialFocus && data.initialFocus.type == "submit" ) {
            focusElement = data.defaultButton;
         } // end else if
         // Give focus to explicitly set item ...
         else if ( data.initialFocus ) {
            focusElement = data.initialFocus;
         } // end else if
         // ... else give fucus to the first non-submit item ...
         else if ( data.firstNonSubmit ) {
            focusElement = data.firstNonSubmit;
         } // end else if
         // ... as a fallback, give focus to the default button
         else {
            focusElement = data.defaultButton;
         } // end else if
      } // end if
      // No default button defined.  Look for initial focus item first.
      else {
         // Set item with explicit focus first ...
         if ( data.initialFocus ) {
            focusElement = data.initialFocus;
         } // end if
         // ... else give focus to the first non-submit item ...
         else if ( data.firstNonSubmit ) {
            focusElement = data.firstNonSubmit;
         } // end else if
         // ... else give focus to the first button ...
         else if ( data.firstSubmit ) {
            focusElement = data.firstSubmit;
         } // end else if
         // ... else give up ...
         else {
            // Punt ... nothing to set focus on ...
         } // end else
      } // end else
   } // end if
   if ( focusElement != null ) {
      setTimeout( "focusElement.focus();", 0 );
   } // end if
// if ( focusElement != null ) {
//    output += "Focus Element = :" + focusElement.tagName + ":" + focusElement.type + ":" + focusElement.value + ":\n";
// } // end if 
// var output = "";
// if ( data.defaultButton != null ) {
//    var defaultButton = data.defaultButton;
//    output += "Default Button = :" + defaultButton.tagName + ":" + defaultButton.type + ":" + defaultButton.value + ":\n";
// } // end if 
// alert( output );
} // setInitialPanelFocus()

/** 
 * 
 * Look at a form determining which components are eligible for initial 
 * focus and what button, if any, is set to the default. 
 * 
 */
function evaluateForm( form, data ) {
   if ( form != null ) {
      var elements = form.elements;
      for ( var i = 0; i < elements.length; i++ ) {
         var element = elements[i];
         var tagName = ( element.tagName == null ) ? "" : element.tagName.toUpperCase();
         // Look at all form elements, ignoring those that are
         // hidden, disabled or readonly
         if ( tagName == TAG_INPUT  ||
              tagName == TAG_BUTTON ||
              tagName == TAG_SELECT ||
              tagName == TAG_TEXTAREA ) {
            var type     = element.type.toUpperCase();
            var value    = element.value.toUpperCase();
            var disabled = element.disabled;
            var readonly = element.readOnly;
            var defaultButton = Boolean( element.getAttribute( DEFAULT_BUTTON ) ).valueOf();
            var defaultFocus  = Boolean( element.getAttribute( DEFAULT_FOCUS ) ).valueOf();
            if ( type != TYPE_HIDDEN && !disabled && !readonly ) {
               if ( data.firstNonSubmit == null &&
                    type != TYPE_SUBMIT && value != VALUE_HELP ) {
                  data.firstNonSubmit = element;
               } // end if
               if ( data.firstSubmit == null && type == TYPE_SUBMIT ) {
                  data.firstSubmit = element;
               } // end if
               if ( data.initialFocus == null && defaultFocus ) {
                  data.initialFocus = element;
               } // end if
               if ( data.defaultButton == null && defaultButton ) {
                  data.defaultButton = element;
               } // end if
               if ( data.allSubmitItems && type != TYPE_SUBMIT ) {
                  data.allSubmitItems = false;
               } // end if
            } // end if
         } // end if
      } // end for
   } // end if
   var anchors = document.getElementsByTagName( TAG_ANCHOR );
   for ( var i = 0; ( i < anchors.length ) && ( data.firstNonSubmit == null ); i++ ) {
      if ( data.firstNonSubmit == null ) {
         if ( Boolean( anchors[i].getAttribute( ATTR_HREF ) ).valueOf() ) {
            data.firstNonSubmit = anchors[i];
         } // end if
      } // end if 
      if ( data.initialFocus == null ) {
         if ( Boolean( anchors[i].getAttribute( DEFAULT_FOCUS ) ).valueOf() ) {
            data.initialFocus = anchors[i];
         } // end if 
      } // end if
   } // end for
} // evaluateForm()

